/* kmaster.cpp
 
  This is my first application using Qt/Kde, it's a simple game called
  KMastermind. I would like to hear what tou think of it, and I'd like
  to know if this program has all the Kde configuration/installation stuff 
  done ok.

  Copyright (C) 1997	Pieter Eendebak	(pte@ddsw.nl)

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

 */


#include "includes.h"

#include "defines.h"
#include "row.h"
#include "dialog.h"
#include "kmaster.moc"

KApplication *kmaster;

/* ##########################
 * #	class TopLevel      #
 * ##########################
 */

void TopLevel::createMenu()
{
	file = new QPopupMenu();
	file->insertItem(klocale->translate("New game"),
		 this, SLOT(newgame()) );
	file->insertItem(klocale->translate("Load game"),
		 this, SLOT(load()) );
	file->insertItem(klocale->translate("Save game"),
		 this, SLOT(save()) );
	file->insertSeparator(-1);
	file->insertItem(klocale->translate("&Quit"),
		 this, SLOT(quitapp()) );
	
	options = new QPopupMenu();
	mlevel = new QPopupMenu();
	mlevel->insertItem(klocale->translate("Easy"), 1);
	mlevel->insertItem(klocale->translate("Normal"), 2);
	mlevel->insertItem(klocale->translate("Hard"), 3);
	mlevel->insertSeparator(-1);
	mlevel->insertItem(klocale->translate("Custom"), 0);
	connect(mlevel, SIGNAL(activated(int) ), this, SLOT(setLevel(int)) );
	mlevel->setCheckable(true);
	mlevel->setItemChecked(0, true);

	options->insertItem(klocale->translate("Configure keys"),
		 this, SLOT(configkeys()) );
	options->insertItem(klocale->translate("&Misc options"),
		 this, SLOT(configmisc()) );
	options->insertItem(klocale->translate("Level"), mlevel);

	help = new QPopupMenu();
	help->insertItem(klocale->translate("&Help"), this, SLOT(helpmenu()) );
	help->insertItem(klocale->translate("&About..."), this, SLOT(about()) );
	
	menu = new KMenuBar(this, "menu");
	menu->insertItem(klocale->translate("&File"), file, 5);
	menu->insertItem(klocale->translate("&Options"), options);
	menu->insertSeparator(-1);
	menu->insertItem(klocale->translate("&Help"), help);

	setMenu(menu);
}

void TopLevel::configmisc()
{
	Misc *m;
	m = new Misc(sound, pixmaps, this, 0);
	if (m->exec() )
	{
		// get all the options...
		config->setGroup("Options");

		if (m->sound->isChecked() )
			sound=1;
		else	sound=0;

		if (m->pixmaps->isChecked() )
			setPixMap(true);
		else	setPixMap(false);
	}
	delete m;

	resizeEvent();
}

void TopLevel::setPixMap(bool yesno)
{
	int i;

	for(i=0;i<rows;i++) rijen[i]->setPixMap( yesno);

	if( yesno )
	{
		pixmaps=1;
		
		int breed = 30*level + 50;
		int hoog = 40*(rows) + 90;

		resize( breed, hoog );
	}
	else
	{
		pixmaps=0;
	}

	resizeEvent();
}

void TopLevel::setLevel(int id)
{
	for(int i=0;i<4;i++) mlevel->setItemChecked(i, id==i);
	
	if(id>0)
	{
		level=id+2;
		rows=6+level;
	}
	else if (id==0)
	{
		
		CustomLevel *l;
		l = new CustomLevel(this, 0);
		if ( l->exec() )	// ok-button pressed
		{
			rows = l->rows;
			level = l->level;
		}
		delete l;
	}

	newgame();
	setPixMap( pixmaps );

}


void TopLevel::resizeEvent( QResizeEvent * )
{
	resizeEvent();
}


void TopLevel::resizeEvent()
{
	QSize s = size();
	menu->setGeometry(0,0, s.width(), (menu->size()).height() );
	play->setGeometry(10,40,100,30);
	check->setGeometry(120,40,100,30);
	
	if ( pixmaps )
	{
		for(int i=0;i<rows;i++)
		{
			rijen[i]->setGeometry(10,80+i*40, s.width()-20, 30);
			rijen[i]->show();
		}
	}
	else
	{
		int d = (s.height()-80)/rows;
	
		if (d < 70 )
		{
		   for(int i=0;i<rows;i++)
		   {
			rijen[i]->setGeometry(10,80+i*d, s.width()-20, d);
			rijen[i]->show();
		   }
		}
		else
		{
		   d = d - 20;
		   for(int i=0;i<rows;i++)
		   {
			rijen[i]->setGeometry(10,80+i*(d+6), s.width()-20, d);
			rijen[i]->show();
		   }
		}
	
	}
}


void TopLevel::helpmenu()
{
	KApplication::getKApplication()->invokeHTMLHelp("","");
}

void TopLevel::load()
{
	int vraag =
	    QMessageBox::information(this, CAPTION,
			klocale->translate("Loading the game will overwrite current game."
			"\nLoad game anyway?"),
			klocale->translate("&Yes"),
			klocale->translate("&No"),
			0, 2 );
	
	config->setGroup("Saved");
	
	if (vraag==0 && config->hasKey("Rows") )
	{
	 
	   rows = config->readNumEntry("Rows");
	   level = config->readNumEntry("Level");

	   for(int i=0;i<rows;i++)
	   {
		for(int j=0;j<level;j++)
		{
		  QString s;
		  s.sprintf("R%dL%d", i, j);
	          rijen[i]->knoppen[j]->
			setColor( config->readNumEntry(s) );
		} 
		rijen[i]->evaluate();
	   }
	}
	resizeEvent();
}

void TopLevel::save()
{
	int vraag =
	QMessageBox::information(this, CAPTION,
			klocale->translate("Saving this game overwrite your previous game."
			"\nSave game anyway?"),
			klocale->translate("&Yes"),
			klocale->translate("&No"),
			0, 2 );
	
	if (vraag==0)
	{
		config->setGroup("Saved");
	
		config->writeEntry("Rows", rows);
		config->writeEntry("Level", level);

		for(int i=0;i<rows;i++)
		{
		   for(int j=0;j<level;j++)
		   {	
			QString s;
			s.sprintf("R%dL%d", i, j);

			config->writeEntry(s,
				 rijen[i]->knoppen[j]->readColor() );
		   }
		}
	}
}

void TopLevel::configkeys()
{
	kKeys->configureKeys(this);
}


void TopLevel::initKeys()
{
	// here we create to shortcuts according to
	// the standard Kde keybinding
	accel = new KStdAccel(config);
	
	kKeys->addKey("about", "CTRL+A");
	kKeys->addKey("helpmenu", accel->help() );
	kKeys->addKey("quit", accel->quit() );
	kKeys->addKey("newgame", accel->openNew() );
	kKeys->addKey("save", accel->save() );
	kKeys->addKey("load", accel->open() );

	kKeys->registerWidget("top", this);
	kKeys->connectFunction("top", "helpmenu", this, SLOT(helpmenu()) );
	kKeys->connectFunction("top", "newgame", this, SLOT(newgame()) );
	kKeys->connectFunction("top", "quit", this, SLOT(quitapp()) );
	kKeys->connectFunction("top", "about", this, SLOT(about()) );
	kKeys->connectFunction("top", "save", this, SLOT(save()) );
	kKeys->connectFunction("top", "load", this, SLOT(load()) );
}

void TopLevel::createCode()
{
	delete [] code;
	code = new int [level];
	for(int i=0;i<level;i++)
	   code[i]= 1+ (int) ( 6.0 * random()/(RAND_MAX+1.0) );
}

bool TopLevel::initSound()
{
	if ( audio==0 )
	{	
		audio = new KAudio();
		connect(this, SIGNAL(requestSound(int)),
			this, SLOT(playSound(int)) );
		if( audio->serverStatus )
		{
			printf("Sound available\n");
			return true;
		}
		else	printf("No sound available\n");
	}
	return false;
}


void TopLevel::endSound()
{
	// if ( audio!=0 ) delete audio;
}

void TopLevel::playSound(int a)
{
	QString dir;
	
	dir.sprintf("%s", base.data() );
	if (sound==1 && audio!=0 )
	{
		switch (a)
		{
		case WINGAME :	dir.append("win.wav");
				break;
		case LOSEGAME :	dir.append("lose.wav");
				break;
		case EVALUATE :	dir.append("evaluate.wav");
				break;
		default	:	return;
				break;
		}
		audio->play( dir.data() );
	}
}

void TopLevel::quitapp()
{
	int vraag =
	QMessageBox::information(this, CAPTION,
			klocale->translate("Do you really want to quit?"),
			klocale->translate("&Yes"),
			klocale->translate("&No"),
			0, 2 );

	if (vraag==0)
	{
		saveConfig();
		kmaster->quit();	
	}	
}


void TopLevel::about()
{
	QString q;
	const char *msg = klocale->translate(
		"\n\nA simple game written in C++ using Qt/Kde."
		"\nWritten by Pieter Eendebak <pte@ddsw.nl>" );

	q.sprintf("%s %s (%s) %s", NAME, VERSION, DATE, msg);
	KMsgBox::message(this, CAPTION, q );
}


void TopLevel::notImpl()
{
	KMsgBox::message(this, CAPTION,
	klocale->translate("Sorry, this function isn't implemented yet...")  );
}

void TopLevel::checkrow()
{
	rijen[currentrow]->evaluate();
}

void TopLevel::rowchange(int id, int state)
{
	if (state==1)
	{
		emit requestSound(WINGAME);
		KMsgBox::message(this, CAPTION,
		klocale->translate("You've found the secret code!") );
	}
	else if ( id==rows && state==0 )
	{
		emit requestSound(LOSEGAME);
		NoCode *p;
		p = new NoCode(this, 0, code, level, pixmaps, pins, base.data() );
		p->exec();
		delete p;
	}
	else if (state==0)
	{
		emit requestSound(EVALUATE);
		rijen[id-1]->setState(1);
		rijen[id]->setState(0);
		if( (id-1)==currentrow ) currentrow++;
	}
}

void TopLevel::newgame()
{
	int i;

	for(i=0;i<rowsave;i++)
		delete rijen[i];
	delete [] rijen;	
	
	// make new secret code
	createCode();	

	rijen = new MasterRow *[rows];
	for(i=0;i<rows;i++)
	{
		rijen[i] = new MasterRow( pixmaps==1, pins, base.data(), this,0, i+1, level, code);
		connect(rijen[i], SIGNAL(changed(int,int)),
			 this, SLOT(rowchange(int,int)) );
	}
	
	rowsave=rows;

	rijen[0]->setState(0);
	currentrow=0;

	resizeEvent();
	repaint();
}


void TopLevel::initConfig()
{
	base.sprintf("%s", (kmaster->kde_datadir()).data() );
	base.append("/kmaster/");

	config = kapp->getConfig();

	config->setGroup("Level");
	rows=config->readNumEntry("rows", 9);
	rowsave=rows;
	level=config->readNumEntry("level", 4);

	config->setGroup("Options");
	sound = config->readNumEntry("sound", 1);

	audio=0;
	initSound();

	pixmaps = config->readNumEntry("pixmaps", 0);

}

void TopLevel::saveConfig()
{
	// save current rows & level
	config->setGroup("Level");
	config->writeEntry("rows", rows);
	config->writeEntry("level", level);

	config->setGroup("Options");
	config->writeEntry("sound", sound);
	config->writeEntry("pixmaps", pixmaps);

	// end sound
	endSound();

	// save everything into files...
	accel->sync();
	config->sync();
}

void TopLevel::setupPixMaps()
{
	QString tmp;
	int i;

	pins = new QPixmap *[7];
	for(i=0;i<7;i++) pins[i] = new QPixmap();

	bool check=true;
	tmp.sprintf("%s%s", base.data(), "none.xpm");
	if ( !pins[0]->load( tmp.data() ) ) check=false;
	tmp.sprintf("%s%s", base.data(), "red.xpm");
	if ( !pins[1]->load( tmp.data() ) ) check=false;
	tmp.sprintf("%s%s", base.data(), "blue.xpm");
	if ( !pins[2]->load( tmp.data() ) ) check=false;
	tmp.sprintf("%s%s", base.data(), "green.xpm");
	if ( !pins[3]->load( tmp.data() ) ) check=false;
	tmp.sprintf("%s%s", base.data(), "yellow.xpm");
	if ( !pins[4]->load( tmp.data() ) ) check=false;
	tmp.sprintf("%s%s", base.data(), "violet.xpm");
	if ( !pins[5]->load( tmp.data() ) ) check=false;
	tmp.sprintf("%s%s", base.data(), "orange.xpm");
	if ( !pins[6]->load( tmp.data() ) ) check=false;

	if (!check)
	{
		KMsgBox::message(this, CAPTION,
			klocale->translate("Could not open pixmaps.")  );
		pixmaps=0;
	}
}


TopLevel:: TopLevel ( QWidget *, const char* name )
	: KTopLevelWidget ( name )
{
	setCaption(CAPTION);
	setMinimumSize(300,400);
	resize(300,650);
	setMaximumSize(500,900);

	initConfig();
	createMenu();
	initKeys();

	setupPixMaps();

	play = new QPushButton(klocale->translate("New game"), this, "new");
	check = new QPushButton(klocale->translate("Check row"), this, "check");
	connect( play, SIGNAL(clicked()), this, SLOT(newgame()) );
	connect( check, SIGNAL(clicked()), this, SLOT(checkrow()) );

	code = new int [rows];
	createCode();
	
	rijen = new MasterRow *[rows];
	rowsave=0;
	
	newgame();
	setPixMap(pixmaps==1);
}

TopLevel::~TopLevel()
{
	// delete [] rijen;
}

/* ##########################
 * #	Main		    #
 * ##########################
 */

int main(int argc, char **argv)
{
	kmaster = new KApplication( argc, argv);
	TopLevel *top;
	top = new TopLevel(0, "kmaster");
	kmaster->setMainWidget(top);
	top->show();
	return kmaster->exec();
}

